%top{
  #include <stdlib.h>
  #include <iostream>
  #include <iomanip>
  #include <vector>
  #include <map>
%}
%{
#include "token_enum.h"
%}
%option flex=false
%option noyywrap
%option never-interactive
%%
0[0-7]+                          { return(OCTAL_TOK); }
[0-9]+                           { return(INTEGER_TOK); }
[0-9]*\.[0-9]+                   { return(FLOAT_TOK); }
[0-9]*\.[0-9]+[Ee][+-][0-9]*     { return(FLOAT_TOK); }
0[xX][0-9a-fA-F]+                { return(HEX_TOK); }
0[bB][01]+                       { return(BIN_TOK); }
'[^\\]+'                         { return(CHAR_TOK); }
'\\[0abfnrtv'\"]'                { return(CTRL_CHAR_TOK); }
'\\[0-7]+'                       { return(OCT_CHAR_TOK); }
'\\x[0-9a-fA-F]+'                { return(HEX_CHAR_TOK); }
\"([^\"\\]|\\.)*\"               { return(STRING_CONST_TOK); }
\$[a-zA-Z_][a-zA-Z0-9_]*         { return(SHELL_VARIABLE_TOK); }
[a-zA-Z_][a-zA-Z0-9_]*           { return(VARIABLE_TOK); }
[a-zA-Z_][a-zA-Z0-9_]*[ \t]*\(   { return(FUNCTION_TOK); }
\/\/                             { return(COMMENT_TOK); }
<<=                              { return(SH_LEFT_ASSIGN_TOK); }
>>=                              { return(SH_RIGHT_ASSIGN_TOK); }
\+\+                             { return(POST_INC_TOK); }
--                               { return(POST_DEC_TOK); }
<<                               { return(SHIFT_LEFT_TOK); }
>>                               { return(SHIFT_RIGHT_TOK); }
<=                               { return(LESS_EQUAL_TOK); }
>=                               { return(GREATER_EQUAL_TOK); }
==                               { return(EQUAL_TOK); }
!=                               { return(NOT_EQUAL_TOK); }
&&                               { return(AND_TOK); }
\|\|                             { return(OR_TOK); }
\*=                              { return(MUL_ASSIGN_TOK); }
\/=                              { return(DIV_ASSIGN_TOK); }
%=                               { return(MOD_ASSIGN_TOK); }
\+=                              { return(ADD_ASSIGN_TOK); }
\-=                              { return(SUB_ASSIGN_TOK); }
&=                               { return(AND_ASSIGN_TOK); }
\|=                              { return(OR_ASSIGN_TOK); }
\^=                              { return(XOR_ASSIGN_TOK); }
-                                { return(UNARY_MINUS_TOK); }
\+                               { return(UNARY_PLUS_TOK); }
\*                               { return(MULTIPLY_TOK); }
\/                               { return(DIVIDE_TOK); }
%                                { return(MODULO_TOK); }
<                                { return(LESS_TOK); }
>                                { return(GREATER_TOK); }
\^                               { return(BIT_XOR_TOK); }
\|                               { return(BIT_OR_TOK); }
@                                { return(ATSIGN_TOK); } 
&                                { return(BIT_AND_TOK); }
=                                { return(ASSIGN_TOK); }
~                                { return(COMPLEMENT_TOK); }
!                                { return(NOT_TOK); }
\?                               { return(QUESTION_TOK); }
:                                { return(COLON_TOK); }
\(                               { return(OPEN_PAREN_TOK); }
\)                               { return(CLOSE_PAREN_TOK); }
\[                               { return(OPEN_BRACE_TOK); }
\]                               { return(CLOSE_BRACE_TOK); }
\{                               { return(OPEN_BRACKET_TOK); }
\}                               { return(CLOSE_BRACKET_TOK); }
;                                { return(SEMICOLON_TOK); }
,                                { return(COMMA_TOK); }
\/\*                             { return(OPEN_COMMENT_TOK); }
\*\/                             { return(CLOSE_COMMENT_TOK); }
#                                { return(COMMENT_TOK); }
\s                               { return(WHITE_SPACE_TOK); }
.                                { return(ERROR_TOK); }

%%
