/////////////////////////////////////////////////////////////////////////////////////
///
/// algebraicfun.cpp
///
/// Acme Software Works, Inc.
///
/// Created January 27, 2004 by Don Dugger
///
/// <PRE>
/// Copyright 2019 Acme Software Works, Inc.
/// 
/// Redistribution and use in source and binary forms, with or without
/// modification, are permitted provided that the following conditions are met:
/// 
/// 1. Redistributions of source code must retain the above copyright notice,
///    this list of conditions and the following disclaimer.
/// 
/// 2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation
///    and/or other materials provided with the distribution.
/// 
/// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
/// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
/// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
/// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
/// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
/// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
/// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
/// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
/// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
/// </PRE>
/////////////////////////////////////////////////////////////////////////////////////

#include "algebraicfun.h"

///////////////////////////////////////////////////////////////

int AlgebraicFun::parseFun(string n,int paren)
{
    try {
        if ( debug_flag ) cout << "argc = " << argc_list[n] << " [" << n << "]" << endl;
        arg_stack.push_back(new ArgElement(paren,argc_list[n]));
        return(arg_stack.back()->fun_argc);
    } catch(ExprError) {
        return(0);
    }

} // End of ParseFun()

///////////////////////////////////////////////////////////////

void AlgebraicFun::commaFun(int paren)
{
    if(arg_stack.back()->fun_argc <= ++(arg_stack.back()->argc)) {
        throw ExprError("Too many arguments",&expr[cur]);
        return;
    } else if(arg_stack.back()->paren != paren) {
        throw ExprError("Miss placed parentheses or comma",&expr[cur]);
        return;
    }

} // End of CommaFun()

///////////////////////////////////////////////////////////////

bool AlgebraicFun::closeParenFun(int paren)
{
    if(arg_stack.empty()) {
        return(false);
    }
    if(arg_stack.back()->paren != paren) {
        return(false);
    }
    if(arg_stack.back()->fun_argc < ++(arg_stack.back()->argc)) {
        throw ExprError("Too many arguments",&expr[cur]);
        return(true);
    }

    delete arg_stack.back();
    arg_stack.pop_back();
    return(true);

} // End of CloseParenFun()

///////////////////////////////////////////////////////////////

void AlgebraicFun::exeFun(string n,vector<ExprElement*>& stack)
{
    ExprElement* operand;
    ExprElement* operand2;
    switch(fun_list[n]) {
    case ABS:
        stack.back()->value = abs(stack.back()->value);
        break;
    case ACOS:
        stack.back()->value = acos(stack.back()->value);
        break;
    case ACOSH:
        stack.back()->value = acosh(stack.back()->value);
        break;
    case ASIN:
        stack.back()->value = asin(stack.back()->value);
        break;
    case ASINH:
        stack.back()->value = asinh(stack.back()->value);
        break;
    case ATAN:
        stack.back()->value = atan(stack.back()->value);
        break;
    case ATAN2:
        operand = stack.back();
        stack.pop_back();
        stack.back()->value = atan2(stack.back()->value,operand->value);
        break;
    case ATANH:
        stack.back()->value = atanh(stack.back()->value);
        break;
    case CBRT:
        stack.back()->value = cbrt(stack.back()->value);
        break;
    case CEIL:
        stack.back()->value = ceil(stack.back()->value);
        break;
    case COPYSIGN:
        operand = stack.back();
        stack.pop_back();
        stack.back()->value = copysign(stack.back()->value,operand->value);
        break;
    case COS:
        stack.back()->value = cos(stack.back()->value);
        break;
    case COSH:
        stack.back()->value = cosh(stack.back()->value);
        break;
    case ERF:
        stack.back()->value = erf(stack.back()->value);
        break;
    case ERFC:
        stack.back()->value = erfc(stack.back()->value);
        break;
    case EXP:
        stack.back()->value = exp(stack.back()->value);
        break;
    case EXP2:
        stack.back()->value = exp2(stack.back()->value);
        break;
    case EXPM1:
        stack.back()->value = expm1(stack.back()->value);
        break;
    case FDIM:
        operand = stack.back();
        stack.pop_back();
        stack.back()->value = fdim(stack.back()->value,operand->value);
        break;
    case FLOOR:
        stack.back()->value = floor(stack.back()->value);
        break;
    case FMA:
        operand2 = stack.back();
        stack.pop_back();
        operand = stack.back();
        stack.pop_back();
        stack.back()->value = fma(stack.back()->value,operand->value,operand2->value);
        break;
    case FMAX:
        operand = stack.back();
        stack.pop_back();
        stack.back()->value = fmax(stack.back()->value,operand->value);
        break;
    case FMIN:
        operand = stack.back();
        stack.pop_back();
        stack.back()->value = fmin(stack.back()->value,operand->value);
        break;
    case FMOD:
        operand = stack.back();
        stack.pop_back();
        stack.back()->value = fmod(stack.back()->value,operand->value);
        break;
    case HYPOT:
        operand = stack.back();
        stack.pop_back();
        stack.back()->value = hypot(stack.back()->value,operand->value);
        break;
    case LDEXP:
        operand = stack.back();
        stack.pop_back();
        stack.back()->value = ldexp(stack.back()->value,operand->value);
        break;
    case LGAMMA:
        stack.back()->value = lgamma(stack.back()->value);
        break;
    case LOG:
        stack.back()->value = log(stack.back()->value);
        break;
    case LOG10:
        stack.back()->value = log10(stack.back()->value);
        break;
    case LOG1P:
        stack.back()->value = log1p(stack.back()->value);
        break;
    case LOG2:
        stack.back()->value = log2(stack.back()->value);
        break;
    case LOGB:
        stack.back()->value = logb(stack.back()->value);
        break;
    case POW:
        operand = stack.back();
        stack.pop_back();
        stack.back()->value = pow(stack.back()->value,operand->value);
        break;
    case REMAINDER:
        operand = stack.back();
        stack.pop_back();
        stack.back()->value = remainder(stack.back()->value,operand->value);
        break;
    case RINT:
        stack.back()->value = rint(stack.back()->value);
        break;
    case ROUND:
        stack.back()->value = round(stack.back()->value);
        break;
    case SIGNBIT:
        stack.back()->value = signbit(stack.back()->value);
        break;
    case SIN:
        stack.back()->value = sin(stack.back()->value);
        break;
    case SINH:
        stack.back()->value = sinh(stack.back()->value);
        break;
    case SQRT:
        stack.back()->value = sqrt(stack.back()->value);
        break;
    case TAN:
        stack.back()->value = tan(stack.back()->value);
        break;
    case TANH:
        stack.back()->value = tanh(stack.back()->value);
        break;
    case TGAMMA:
        stack.back()->value = tgamma(stack.back()->value);
        break;
    case TRUNC:
        stack.back()->value = trunc(stack.back()->value);
        break;
    case XRT:
        operand = stack.back();
        stack.pop_back();
        stack.back()->value = pow(stack.back()->value,1/operand->value);
        break;
    }

} // End of ExeFun()


///////////////////////////////////////////////////////////////
///////////////////// END OF FILE /////////////////////////////
///////////////////////////////////////////////////////////////
