/////////////////////////////////////////////////////////////////////////////////////
///
/// algebraicfun.h
///
/// Acme Software Works, Inc.
///
/// Created January 27, 2004 by Don Dugger
///
/// <PRE>
/// Copyright 2019 Acme Software Works, Inc.
/// 
/// Redistribution and use in source and binary forms, with or without
/// modification, are permitted provided that the following conditions are met:
/// 
/// 1. Redistributions of source code must retain the above copyright notice,
///    this list of conditions and the following disclaimer.
/// 
/// 2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation
///    and/or other materials provided with the distribution.
/// 
/// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
/// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
/// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
/// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
/// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
/// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
/// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
/// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
/// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
/// </PRE>
/////////////////////////////////////////////////////////////////////////////////////

#ifndef _algebraicfun_h_
#define _algebraicfun_h_

/////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string>
#include <cmath>
#include <vector>
#include <map>

#include "algebraicexpr.h"

/////////////////////////////////////////////////////////////////////////////////////
/// Classes

class AlgebraicFun : public AlgebraicExpr {
    /// Function enumerators 
    enum MATH_FUNCTION {
        ABS,
        ACOS,
        ACOSH,
        ASIN,
        ASINH,
        ATAN,
        ATAN2,
        ATANH,
        CBRT,
        CEIL,
        COPYSIGN,
        COS,
        COSH,
        ERF,
        ERFC,
        EXP,
        EXP2,
        EXPM1,
        FDIM,
        FLOOR,
        FMA,
        FMAX,
        FMIN,
        FMOD,
        HYPOT,
        LDEXP,
        LGAMMA,
        LOG,
        LOG10,
        LOG1P,
        LOG2,
        LOGB,
        POW,
        REMAINDER,
        RINT,
        ROUND,
        SIGNBIT,
        SIN,
        SINH,
        SQRT,
        TAN,
        TANH,
        TGAMMA,
        TRUNC,
        XRT
    };

    /// Argument Element
    /// A class for keeping track of parentheses and arguments
    class ArgElement {
    public:
        ArgElement(int pl,int fa) { paren = pl; fun_argc = fa; argc = 0; };
        int paren;     // Parentheses depth
        int argc;      // Current argument count
        int fun_argc;  // This function's required number of arguments
    };

    /// The argument element stack
    vector<ArgElement*> arg_stack;
    /// For translating function tokens to enumerator 
    map<string,MATH_FUNCTION> fun_list {
                { "abs(",       ABS       },
                { "acos(",      ACOS      },
                { "acosh(",     ACOSH     },
                { "asin(",      ASIN      },
                { "asinh(",     ASINH     },
                { "atan(",      ATAN      },
                { "atan2(",     ATAN2     },
                { "atanh(",     ATANH     },
                { "cbrt(",      CBRT      },
                { "ceil(",      CEIL      },
                { "copysign(",  COPYSIGN  },
                { "cos(",       COS       },
                { "cosh(",      COSH      },
                { "erf(",       ERF       },
                { "erfc(",      ERFC      },
                { "exp(",       EXP       },
                { "exp2(",      EXP2      },
                { "expm1(",     EXPM1     },
                { "fdim(",      FDIM      },
                { "floor(",     FLOOR     },
                { "fma(",       FMA       },
                { "fmax(",      FMAX      },
                { "fmin(",      FMIN      },
                { "fmod(",      FMOD      },
                { "hypot(",     HYPOT     },
                { "ldexp(",     LDEXP     },
                { "lgamma(",    LGAMMA    },
                { "log(",       LOG       },
                { "log10(",     LOG10     },
                { "log1p(",     LOG1P     },
                { "log2(",      LOG2      },
                { "logb(",      LOGB      },
                { "pow(",       POW       },
                { "remainder(", REMAINDER },
                { "rint(",      RINT      },
                { "round(",     ROUND     },
                { "signbit(",   SIGNBIT   },
                { "sin(",       SIN       },
                { "sinh(",      SINH      },
                { "sqrt(",      SQRT      },
                { "tan(",       TAN       },
                { "tanh(",      TANH      },
                { "tgamma(",    TGAMMA    },
                { "trunc(",     TRUNC     },
                { "xrt(",       XRT       }
    };
    /// The function's number of required arguments list
    map<string,int> argc_list {
                { "abs(",       1 },
                { "acos(",      1 },
                { "acosh(",     1 },
                { "asin(",      1 },
                { "asinh(",     1 },
                { "atan(",      1 },
                { "atan2(",     2 },
                { "atanh(",     1 },
                { "cbrt(",      1 },
                { "ceil(",      1 },
                { "copysign(",  2 },
                { "cos(",       1 },
                { "cosh(",      1 },
                { "dgr(",       1 },
                { "erf(",       1 },
                { "erfc(",      1 },
                { "exp(",       1 },
                { "exp2(",      1 },
                { "expm1(",     1 },
                { "fdim(",      2 },
                { "floor(",     1 },
                { "fma(",       3 },
                { "fmax(",      2 },
                { "fmin(",      2 },
                { "fmod(",      2 },
                { "hypot(",     2 },
                { "ldexp(",     2 },
                { "lgamma(",    1 },
                { "log(",       1 },
                { "log10(",     1 },
                { "log1p(",     1 },
                { "log2(",      1 },
                { "logb(",      1 },
                { "pow(",       2 },
                { "rad(",       1 },
                { "remainder(", 2 },
                { "rint(",      1 },
                { "round(",     1 },
                { "signbit(",   1 },
                { "sin(",       1 },
                { "sinh(",      1 },
                { "sqrt(",      1 },
                { "tan(",       1 },
                { "tanh(",      1 },
                { "tgamma(",    1 },
                { "trunc(",     1 },
                { "xrt(",       2 }
    };

public:
    /// @note The constructors simply invoke the base class constructors
    /// 
    /// This constructor is used to create an AlgebraicFun object
    /// that when used in the null form will not allow assignments
    /// and will allocate an internal variable list. It can optionally
    /// allow assignments when it is invoked with argument "true".
    /// @param as   The optional assignments flag.
    /// @brief  The null constructor.
    AlgebraicFun(bool as = false) : AlgebraicExpr(as) {};
    /// This constructor is used to create an AlgebraicFun object
    /// that will use the variable list which it is given as an 
    /// argument and by default will allow assignments. However
    /// if given a second argument of "false" will not allow assignments.
    /// @param var  The external variable list.
    /// @param as   The optional assignments flag.
    /// @brief  The constructor with the external variable list.
    AlgebraicFun(map<string,double>& var,bool allow = true) : AlgebraicExpr(var,allow) {};
    /// The destructor that must, if allocated, free the internal variable list.
    ~AlgebraicFun() {};

    /// Create an ArgElemet with parentheses depth and the function's number
    /// of required arguments. And place it in the argument list.
    /// @param n     Text of the function name.
    /// @param paren The current parentheses depth
    /// @return The functions required number of arguments or on error zero.
    /// @brief Create an ArgElemet and place it on the argument list.
    int parseFun(string n,int paren);
    /// Check that the commas are correctly placed.
    /// @param paren    The parentheses depth.
    /// @brief  Check commas.
    void commaFun(int paren);
    /// Handle the closing parentheses. Check that there are the correct number
    /// of arguments.
    /// @paren  The parentheses depth.
    /// @return true if there are no errors and false otherwise. 
    /// @brief  Handle the closing parentheses.
    bool closeParenFun(int paren);
    ///  Execute the function and enter the return value in the expression
    ///  element on the top of the stack. If there are more then one argument
    ///  remove them from the stack.
    ///  @param n     Text of the function name.
    ///  @param stack The expression element stack.
    ///  @brief Execute the function.
    void exeFun(string n,vector<ExprElement*>& stack);
};


///////////////////////////////////////////////////////////////////////

#endif // _algebraicfun_h_

