/////////////////////////////////////////////////////////////////////////////////////
///
/// Acme Software Works, Inc.
///
/// Created January 27, 2004 by Don Dugger
///
/// <PRE>
/// Copyright 2019 Acme Software Works, Inc.
/// 
/// Redistribution and use in source and binary forms, with or without
/// modification, are permitted provided that the following conditions are met:
/// 
/// 1. Redistributions of source code must retain the above copyright notice,
///    this list of conditions and the following disclaimer.
/// 
/// 2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation
///    and/or other materials provided with the distribution.
/// 
/// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
/// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
/// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
/// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
/// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
/// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
/// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
/// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
/// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
/// </PRE>
/////////////////////////////////////////////////////////////////////////////////////

#include <string>
#include <iostream>
#include <sstream>
#include <vector>
#include <map>

/////////////////////////////////////////////////////////////*/
//
#include "algebraicfun.h"
// 
using namespace std;
//
/////////////////////////////////////////////////////////////*/
//
// Change the defualt termination to exit
//
void terminator() {
    exit(-1);
}

void (*old_terminate)() = set_terminate(terminator);


//////////////////////////////////////////////////////////////
//
// Split a string at the delimiter
//
vector<string> split( string str, string delimiter) 
{
    size_t start = 0;
    size_t end;
    size_t delimiter_len = delimiter.length();
    string segment;
    vector<string> list;

    while ((end = str.find(delimiter, start)) != string::npos) {
        segment = str.substr(start, end - start);
        list.push_back(segment);
        start = end + delimiter_len;
    }
    list.push_back(str.substr(start));
    return list;

} // End of split ()


//////////////////////////////////////////////////////////////
//
// main
//
//

int main(int argc,char* argv[])
{
    string input_str;
    double x = 0;
    map<string,double> var_list;
    vector<string> expressions;

    enum COMMANDS {
        DEFAULT,   // Use up 0
        QUIT,
        HELP,
        VARS,
        DEBUG
    };

    map<string,COMMANDS> cmd_list {
        { ".",  QUIT    },
        { "?",  HELP    },
        { "=",  VARS    },
        { "!0", DEBUG0  },
        { "!1", DEBUG1  },
        { "!2", DEBUG2  }

    };


    {
        AlgebraicFun math(var_list);

        math.solve("PI=3.141592653589793");
        math.solve("E=2.718281828459045");
        math.solve("PHI=1.61803398875");

        for (;;) {
            getline(cin,input_str);
            expressions = split(input_str,";");
            for ( auto &expression : expressions ) {
                if ( expression.size() == 0 ) exit(0);
                cout << "[" << expression << "]" << endl;
                switch(cmd_list[expression]) {
                    case QUIT:
                        exit(0);
                    case HELP:
                        cout << "Enter the character followed by Return" << endl;
                        cout << ".  Quit" << endl;
                        cout << "?  Help (This)" << endl;
                        cout << "=  Variable List" << endl;
                        cout << "!0 Cancel debug mode" << endl;
                        cout << "!1 Debug mode 1" << endl;
                        cout << "!2 Debug mode 2" << endl;

                        break;
                    case VARS:
                        cout << "---------Variable_List----------" << endl;
                        for (auto const& x : var_list) {
                                cout << x.first  // string (key)
                                      << ": \t"
                                      << x.second // string's value
                                      << endl ;
                        }
                        break;
                    case DEBUG0:
                        math.debug(0);
                        break;
                    case DEBUG1 :
                        math.debug(1);
                        break;
                    case DEBUG2 :
                        math.debug(2);
                        break;
                    default:
                        x = math.solve(expression);
                        cout << "RESULT> " << x << endl;
                }
            }
        }
    }

    exit(0);

} // End of main()

//////////////////////////////////////////////////////////////
///////////////////// END OF FILE ////////////////////////////
//////////////////////////////////////////////////////////////
